#include <stm32f10x_gpio.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_spi.h>
#include <stm32f10x_exti.h>
#include <misc.h>

#include <Usart.h>
#include <NRF24L01.h>
#include <Delay.h>


#define RX_PAYLOAD           18   // nRF24L01 TX payload length


uint8_t buf[RX_PAYLOAD];
uint8_t have_data;


static uint8_t CRC7_one(uint8_t t, uint8_t data) {
	const uint8_t g = 0x89;
	uint8_t i;

	t ^= data;
	for (i = 0; i < 8; i++) {
		if (t & 0x80) t ^= g;
		t <<= 1;
	}

	return t;
}

uint8_t CRC7_buf(const uint8_t * p, uint8_t len) {
	uint8_t j,crc = 0;

	for (j = 0; j < len; j++) crc = CRC7_one(crc,p[j]);

	return crc >> 1;
}

void EXTI15_10_IRQHandler(void) {
	uint8_t i;
	uint8_t status;

	USART_SendStr("---IRQ-->>>\n");
	if (EXTI_GetITStatus(EXTI_Line10) != RESET) {
		for (i = 0; i < RX_PAYLOAD; i++) buf[i] = 0x00;
		USART_SendStr("RXPacket: ");
		status = nRF24_RXPacket(buf,RX_PAYLOAD);
		nRF24_ClearIRQFlags();
		USART_SendHex8(status);

		if (status == 0x0E) {
			USART_SendStr(" => FIFO Empty (fake alarm)\n");
		} else {
			have_data = 1;
			USART_SendChar('\n');
		}

		GPIOC->ODR ^= GPIO_Pin_9; // Toggle LED

		nRF24_RXMode(RX_PAYLOAD);
		nRF24_ClearIRQFlags();

		EXTI_ClearITPendingBit(EXTI_Line10);
	}

	USART_SendStr("---IRQ--<<<\n");
}


int main(void) {
	
		int16_t temp;
	uint8_t CRC7;
	uint16_t vrefint;
	uint16_t LSI_freq;
	uint8_t TR1, TR2, TR3;
	uint8_t DR1, DR2, DR3;
	uint8_t hours, minutes, seconds;
	uint8_t day, month, year;
	
	unsigned char i,v;
	
void EXTI_Config();
	
	// Init LED pins
	GPIO_InitTypeDef PORT;
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC,ENABLE);
	PORT.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_8;
	PORT.GPIO_Mode = GPIO_Mode_Out_PP;
	PORT.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC,&PORT);

	USART_init(115200);
	USART_SendStr("\nSTM32F103RET6 is online.\n");

	nRF24_init();

	if (nRF24_Check() != 0) {
		USART_SendStr("Got wrong answer from SPI device.\n");
		USART_SendStr("MCU is now halt.\n");
		while(1);
	}

	

	for (i = 0; i < 0x1D; i++) {
		USART_SendHex8(i);
		USART_SendStr("=");
		v = nRF24_ReadReg(i);
		USART_SendHex8(v);
		USART_SendStr("  ");
	}
	USART_SendChar('\n');

	nRF24_RXMode(RX_PAYLOAD);
	nRF24_ClearIRQFlags();


	have_data = 0;
	while(1) {
		while(!have_data);

		have_data = 0;
		CRC7 = CRC7_buf(&buf[0],RX_PAYLOAD-1);

		USART_SendStr("RX=["); USART_SendBufHex((char*)&buf[0],RX_PAYLOAD);
		USART_SendStr("] CRC=("); USART_SendInt(buf[RX_PAYLOAD-1]);
		USART_SendStr("|"); USART_SendInt(CRC7);
		USART_SendStr((buf[RX_PAYLOAD-1] == CRC7) ? ") => ok\n" : ") => bad\n");
		USART_SendStr("Temperature: ");
		temp = (buf[0] << 8) | buf[1];
		USART_SendInt(temp / 10); USART_SendChar('.');
		temp %= 10;
		if (temp < 0) temp *= -1;
		USART_SendInt(temp % 10); USART_SendStr("C\n");

		USART_SendStr("Packet: #");
		USART_SendInt((uint32_t)((buf[2] << 24)|(buf[3] << 16)|(buf[4] << 8)|(buf[5])));
		USART_SendChar('\n');

		vrefint = (buf[6] << 8) + buf[7];
		USART_SendStr("Vcc: ");
		USART_SendInt(vrefint / 100);
		USART_SendChar('.');
		USART_SendInt0(vrefint % 100);
		USART_SendStr("V\n");

		LSI_freq = (buf[14] << 8) + buf[15];
		USART_SendStr("LSI: ");
		USART_SendInt(LSI_freq);
		USART_SendStr("Hz\n");

		USART_SendStr("OBSERVE_TX:\n  ");
		USART_SendHex8(buf[16] >> 4); USART_SendStr(" pckts lost\n  ");
		USART_SendHex8(buf[16] & 0x0F); USART_SendStr(" retries\n");

		TR1 = buf[8];
		TR2 = buf[9];
		TR3 = buf[10];
		DR1 = buf[11];
		DR2 = buf[12];
		DR3 = buf[13];
		seconds = ((TR1 >> 4) * 10) + (TR1 & 0x0F);
		minutes = ((TR2 >> 4) * 10) + (TR2 & 0x0F);
		hours   = (((TR3 & 0x30) >> 4) * 10) + (TR3 & 0x0F);
	    day   = ((DR1 >> 4) * 10) + (DR1 & 0x0F);
	    //dow   = DR2 >> 5;
	    month = (((DR2 & 0x1F) >> 4) * 10) + (DR2 & 0x0F);
	    year  = ((DR3 >> 4) * 10) + (DR3 & 0x0F);

		USART_SendStr("Uptime: ");
        USART_SendInt0(hours); USART_SendChar(':');
        USART_SendInt0(minutes); USART_SendChar(':');
        USART_SendInt0(seconds); USART_SendChar(' ');
        USART_SendInt0(day); USART_SendChar('.');
        USART_SendInt0(month); USART_SendStr(".20");
        USART_SendInt0(year); USART_SendChar('\n');
	}
}
