#include "string.h"
#include <stm32f10x_gpio.h>
#include <stm32f10x_rcc.h>
#include <stm32f10x_usart.h>
#include <Usart.h>


void USART_init(uint32_t baudrate) {
	GPIO_InitTypeDef PORT;
	USART_InitTypeDef USART;

	// U(S)ART init
#if _USART_PORT == 1
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1,ENABLE);
#elif _USART_PORT == 2
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA,ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2,ENABLE);
#elif _USART_PORT == 3
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB,ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3,ENABLE);
#endif

	PORT.GPIO_Pin = USART_TX_PIN;
	PORT.GPIO_Speed = GPIO_Speed_50MHz;
	PORT.GPIO_Mode = GPIO_Mode_AF_PP; // TX as AF with Push-Pull
	GPIO_Init(USART_GPIO_PORT_TX,&PORT);
	PORT.GPIO_Pin = USART_RX_PIN;
	PORT.GPIO_Speed = GPIO_Speed_50MHz;
	PORT.GPIO_Mode = GPIO_Mode_IN_FLOATING; // RX as in without pull-up
	GPIO_Init(USART_GPIO_PORT_RX,&PORT);

	
	USART.USART_BaudRate = baudrate;
	USART.USART_HardwareFlowControl = USART_HardwareFlowControl_None; // No flow control
	USART.USART_Mode = USART_Mode_Rx | USART_Mode_Tx; // RX+TX mode
	USART.USART_WordLength = USART_WordLength_8b; // 8-bit frame
	USART.USART_Parity = USART_Parity_No; // No parity check
	USART.USART_StopBits = USART_StopBits_1; // 1 stop bit
	USART_Init(USART_PORT,&USART);
	USART_Cmd(USART_PORT,ENABLE);
}

void USART_SendChar(char ch) {
	while (!USART_GetFlagStatus(USART_PORT,USART_FLAG_TC)); // wait for "Transmission Complete" flag cleared
	USART_SendData(USART_PORT,ch);
}

void USART_SendInt(int32_t num) {
	char str[10]; // 10 chars max for INT32_MAX
	int i = 0;
	if (num < 0) {
		USART_SendChar('-');
		num *= -1;
	}
	do str[i++] = num % 10 + '0'; while ((num /= 10) > 0);
	for (i--; i >= 0; i--) USART_SendChar(str[i]);
}

void USART_SendInt0(int32_t num) {
	char str[10]; // 10 chars max for INT32_MAX
	int i = 0;
	if (num < 0) {
		USART_SendChar('-');
		num *= -1;
	}
	if ((num < 10) && (num >= 0)) USART_SendChar('0');
	do str[i++] = num % 10 + '0'; while ((num /= 10) > 0);
	for (i--; i >= 0; i--) USART_SendChar(str[i]);
}

void USART_SendHex8(uint16_t num) {
	USART_SendChar(HEX_CHARS[(num >> 4)   % 0x10]);
	USART_SendChar(HEX_CHARS[(num & 0x0f) % 0x10]);
}

void USART_SendHex16(uint16_t num) {
	uint8_t i;
	for (i = 12; i > 0; i -= 4) USART_SendChar(HEX_CHARS[(num >> i) % 0x10]);
	USART_SendChar(HEX_CHARS[(num & 0x0f) % 0x10]);
}

void USART_SendHex32(uint32_t num) {
	uint8_t i;
	for (i = 28; i > 0; i -= 4)	USART_SendChar(HEX_CHARS[(num >> i) % 0x10]);
	USART_SendChar(HEX_CHARS[(num & 0x0f) % 0x10]);
}

void USART_SendStr(char *str) {
	while (*str) USART_SendChar(*str++);
}

void USART_SendBuf(char *buf, uint16_t bufsize) {
	uint16_t i;
	for (i = 0; i < bufsize; i++) USART_SendChar(*buf++);
}

void USART_SendBufPrintable(char *buf, uint16_t bufsize, char subst) {
	uint16_t i;
	char ch;
	for (i = 0; i < bufsize; i++) {
		ch = *buf++;
		USART_SendChar(ch > 32 ? ch : subst);
	}
}

void USART_SendBufHex(char *buf, uint16_t bufsize) {
	uint16_t i;
	char ch;
	for (i = 0; i < bufsize; i++) {
		ch = *buf++;
		USART_SendChar(HEX_CHARS[(ch >> 4)   % 0x10]);
		USART_SendChar(HEX_CHARS[(ch & 0x0f) % 0x10]);
	}
}

void USART_SendBufHexFancy(char *buf, uint16_t bufsize, uint8_t column_width, char subst) {
	char buffer[512];
	uint16_t i = 0,len,pos;

	while (i < bufsize) {
		// Line number
		USART_SendHex16(i);
		USART_SendChar(':'); USART_SendChar(' '); // Faster and less code than USART_SendStr(": ");

		// Copy one line
		if (i+column_width >= bufsize) len = bufsize - i; else len = column_width;
		memcpy(buffer,&buf[i],len);

		// Hex data
		pos = 0;
		while (pos < len) USART_SendHex8(buffer[pos++]);
		USART_SendChar(' ');

		// Raw data
		pos = 0;
		do USART_SendChar(buffer[pos] > 32 ? buffer[pos] : subst); while (++pos < len);
		USART_SendChar('\n');

		i += len;
	}
}
